/**
 * Product panel control for AffiliateRatingType product fields.
 *
 * @module package/quiqqer/affiliate-products/bin/js/backend/controls/fields/AffiliateRatingType
 * @author www.pcsg.de (Patrick Müller)
 */
define('package/quiqqer/affiliate-products/bin/js/backend/controls/fields/AffiliateRatingType', [

    'qui/controls/Control',
    'qui/controls/loader/Loader',
    'package/quiqqer/products/bin/Fields',

    'qui/utils/Form',
    'Locale',
    'Mustache',

    'text!package/quiqqer/affiliate-products/bin/js/backend/controls/fields/AffiliateRatingType.html',
    'text!package/quiqqer/affiliate-products/bin/js/backend/controls/fields/AffiliateRatingType.Criterion.html',
    'css!package/quiqqer/affiliate-products/bin/js/backend/controls/fields/AffiliateRatingType.css'

], function (QUIControl, QUILoader, ProductFields, QUIFormUtils, QUILocale, Mustache, template, templateCriterion) {
    "use strict";

    const lg = 'quiqqer/affiliate-products';

    const RATING_TYPE_PERCENT = 1;
    const RATING_TYPE_POINTS  = 2;
    const RATING_TYPE_BOOLEAN = 3;
    const RATING_TYPE_YES_NO  = 4;
    const RATING_TYPE_TEXT    = 5;

    return new Class({
        Extends: QUIControl,
        Type   : 'package/quiqqer/affiliate-products/bin/js/backend/controls/fields/AffiliateRatingType',

        Binds: [
            '$onImport',
            '$getOptions',
            '$onSelectChange',
            '$setValue',
            'enable',
            'disable'
        ],

        initialize: function (options) {
            this.parent(options);

            this.$Select        = null;
            this.$fieldId       = null;
            this.$QuantityInput = null;
            this.$Input         = null;

            this.Loader = new QUILoader();

            this.addEvents({
                onImport: this.$onImport
            });
        },

        /**
         * event : on import
         */
        $onImport: function () {
            this.$Input      = this.getElm();
            this.$Input.type = 'hidden';

            this.$fieldId = this.$Input.get('name').split('-')[1];

            this.$Elm = new Element('div', {
                'class': 'quiqqer-affiliate-products-AffiliateRatingType field-container-field'
            }).inject(this.$Input, 'after');

            this.Loader.inject(this.$Elm);
            this.Loader.show();

            ProductFields.getFieldOptions(this.$fieldId).then((Options) => {
                const maxPoints = parseInt(Options.totalRatingMaxPoints);

                this.$Elm.set('html', Mustache.render(template, {
                    headerTotalRating         : QUILocale.get(lg, 'AffiliateRatingTypeFrontendView.totalRating.title'),
                    labelTotalRatingPercent   : QUILocale.get(lg, 'controls.AffiliateRatingType.tpl.labelTotalRatingPercent'),
                    labelTotalRatingPoints    : QUILocale.get(lg, 'controls.AffiliateRatingType.tpl.labelTotalRatingPoints', {
                        maxPoints: maxPoints
                    }),
                    labelTotalRatingBoolean   : QUILocale.get(lg, 'controls.AffiliateRatingType.tpl.labelTotalRatingBoolean'),
                    labelBooleanOptionPositive: QUILocale.get(lg, 'controls.AffiliateRatingType.tpl.labelBooleanOptionPositive'),
                    labelBooleanOptionNeutral : QUILocale.get(lg, 'controls.AffiliateRatingType.tpl.labelBooleanOptionNeutral'),
                    labelBooleanOptionNegative: QUILocale.get(lg, 'controls.AffiliateRatingType.tpl.labelBooleanOptionNegative'),
                    labelTotalRatingComment   : QUILocale.get(lg, 'controls.AffiliateRatingType.tpl.labelTotalRatingComment'),

                    labelTotalRatingYesNo: QUILocale.get(lg, 'controls.AffiliateRatingType.tpl.labelTotalRatingYesNo'),
                    labelYesNoOptionYes  : QUILocale.get(lg, 'controls.AffiliateRatingType.tpl.labelYesNoOptionYes'),
                    labelYesNoOptionNo   : QUILocale.get(lg, 'controls.AffiliateRatingType.tpl.labelYesNoOptionNo'),
                    labelTotalRatingText : QUILocale.get(lg, 'controls.AffiliateRatingType.tpl.labelTotalRatingText'),

                    maxPoints: maxPoints,

                    totalRatingTypePercent: parseInt(Options.totalRatingType) === RATING_TYPE_PERCENT,
                    totalRatingTypePoints : parseInt(Options.totalRatingType) === RATING_TYPE_POINTS,
                    totalRatingTypeBoolean: parseInt(Options.totalRatingType) === RATING_TYPE_BOOLEAN,
                    totalRatingTypeYesNo  : parseInt(Options.totalRatingType) === RATING_TYPE_YES_NO,
                    totalRatingTypeText   : parseInt(Options.totalRatingType) === RATING_TYPE_TEXT
                }));

                const CriteriaContainer = this.$Elm.getElement('.quiqqer-affiliate-products-AffiliateRatingType-criteria');
                const currentLang       = QUILocale.getCurrent();

                for (let [criterionId, Criterion] of Object.entries(Options.criteria)) {
                    const Title        = JSON.decode(Criterion.title);
                    let criterionTitle = criterionId;

                    if (typeof Title[currentLang] !== 'undefined') {
                        criterionTitle = Title[currentLang];
                    }

                    const maxPoints = parseInt(Criterion.ratingMaxPoints);

                    new Element('div', {
                        'class'  : 'quiqqer-affiliate-products-AffiliateRatingType-criteria-entry',
                        'data-id': criterionId,
                        html     : Mustache.render(templateCriterion, {
                            criterion: criterionTitle,

                            labelRatingPercent: QUILocale.get(lg, 'controls.AffiliateRatingType.tpl.labelRatingPercent'),
                            labelRatingPoints : QUILocale.get(lg, 'controls.AffiliateRatingType.tpl.labelRatingPoints', {
                                maxPoints: maxPoints
                            }),

                            labelRatingBoolean: QUILocale.get(lg, 'controls.AffiliateRatingType.tpl.labelRatingBoolean'),
                            labelRatingComment: QUILocale.get(lg, 'controls.AffiliateRatingType.tpl.labelRatingComment'),

                            labelBooleanOptionPositive: QUILocale.get(lg, 'controls.AffiliateRatingType.tpl.labelBooleanOptionPositive'),
                            labelBooleanOptionNeutral : QUILocale.get(lg, 'controls.AffiliateRatingType.tpl.labelBooleanOptionNeutral'),
                            labelBooleanOptionNegative: QUILocale.get(lg, 'controls.AffiliateRatingType.tpl.labelBooleanOptionNegative'),
                            labelRatingYesNo          : QUILocale.get(lg, 'controls.AffiliateRatingType.tpl.labelRatingYesNo'),
                            labelRatingText           : QUILocale.get(lg, 'controls.AffiliateRatingType.tpl.labelRatingText'),

                            labelYesNoOptionYes: QUILocale.get(lg, 'controls.AffiliateRatingType.tpl.labelYesNoOptionYes'),
                            labelYesNoOptionNo : QUILocale.get(lg, 'controls.AffiliateRatingType.tpl.labelYesNoOptionNo'),

                            maxPoints: maxPoints,

                            ratingTypePercent: parseInt(Criterion.ratingType) === RATING_TYPE_PERCENT,
                            ratingTypePoints : parseInt(Criterion.ratingType) === RATING_TYPE_POINTS,
                            ratingTypeBoolean: parseInt(Criterion.ratingType) === RATING_TYPE_BOOLEAN,
                            ratingTypeYesNo  : parseInt(Criterion.ratingType) === RATING_TYPE_YES_NO,
                            ratingTypeText   : parseInt(Criterion.ratingType) === RATING_TYPE_TEXT
                        })
                    }).inject(CriteriaContainer);
                }

                if (this.$Input.value !== '') {
                    const Settings = JSON.decode(this.$Input.value);

                    const TotalRatingContainer = this.$Elm.getElement(
                        '.quiqqer-affiliate-products-AffiliateRatingType-totalRating'
                    );

                    QUIFormUtils.setDataToNode(Settings, TotalRatingContainer);

                    const criteriaNodes = this.$Elm.getElements(
                        '.quiqqer-affiliate-products-AffiliateRatingType-criteria-entry'
                    );

                    for (let i = 0, len = criteriaNodes.length; i < len; i++) {
                        const CriteriaNode = criteriaNodes[i];
                        const criteriaId   = CriteriaNode.get('data-id');

                        if (criteriaId in Settings.criteria) {
                            QUIFormUtils.setDataToNode(Settings.criteria[criteriaId], CriteriaNode);
                        }
                    }
                }

                this.$Elm.getElements('input').addEvent('change', this.$setValue);
                this.$Elm.getElements('select').addEvent('change', this.$setValue);
                this.$Elm.getElements('textarea').addEvent('change', this.$setValue);

                this.Loader.hide();
            });
        },

        /**
         * Event: on value select
         *
         * @param {DOMEvent} event
         */
        $onSelectChange: function (event) {
            var Option = this.$Select.getElement('option[value="' + event.target.value + '"]');

            this.$setValue();

            if (!parseInt(Option.get('data-input'))) {
                this.$QuantityInput.setStyle('display', 'none');
                return;
            }

            this.$QuantityInput.setStyle('display', '');
            this.$QuantityInput.focus();
        },

        /**
         * Set field value to input
         */
        $setValue: function () {
            const TotalRatingContainer = this.$Elm.getElement(
                '.quiqqer-affiliate-products-AffiliateRatingType-totalRating'
            );

            const FieldSettings = QUIFormUtils.getDataFromNode(TotalRatingContainer);

            FieldSettings.criteria = {};

            const criteriaNodes = this.$Elm.getElements(
                '.quiqqer-affiliate-products-AffiliateRatingType-criteria-entry'
            );

            for (let i = 0, len = criteriaNodes.length; i < len; i++) {
                const CriteriaNode                                  = criteriaNodes[i];
                FieldSettings.criteria[CriteriaNode.get('data-id')] = QUIFormUtils.getDataFromNode(CriteriaNode);
            }

            this.$Input.value = JSON.encode(FieldSettings);
        },

        /**
         * Return the current value
         *
         * @returns {Object}
         */
        getValue: function () {
            return this.$Input.value;
        },

        /**
         * Enable control
         */
        enable: function () {
            this.$Elm.getElements('input').disabled    = false;
            this.$Elm.getElements('select').disabled   = false;
            this.$Elm.getElements('textarea').disabled = false;
        },

        /**
         * Disable control
         */
        disable: function () {
            this.$Elm.getElements('input').disabled    = true;
            this.$Elm.getElements('select').disabled   = true;
            this.$Elm.getElements('textarea').disabled = true;
        }
    });
});
