/**
 * @module package/quiqqer/affiliate-products/bin/js/backend/controls/fields/ProductAttributeList
 * @author www.pcsg.de (Patrick Müller)
 */
define('package/quiqqer/affiliate-products/bin/js/backend/controls/fields/AffiliateRatingTypeSettings', [

    'qui/QUI',
    'qui/controls/Control',
    'qui/controls/windows/Confirm',
    'Locale',
    'controls/grid/Grid',

    'qui/utils/Form',

    'Mustache',

    'text!package/quiqqer/affiliate-products/bin/js/backend/controls/fields/AffiliateRatingTypeSettings.html',
    'text!package/quiqqer/affiliate-products/bin/js/backend/controls/fields/AffiliateRatingTypeSettings.Criterion.html',
    'css!package/quiqqer/affiliate-products/bin/js/backend/controls/fields/AffiliateRatingTypeSettings.css'

], function (QUI, QUIControl, QUIConfirm, QUILocale, Grid, QUIFormUtils, Mustache, template, templateCriterion) {
    "use strict";

    var lg = 'quiqqer/affiliate-products';

    return new Class({
        Extends: QUIControl,
        Type   : 'package/quiqqer/affiliate-products/bin/js/backend/controls/fields/AffiliateRatingTypeSettings',

        Binds: [
            'update',
            'openAddDialog',
            'openEditDialog',
            'openRemoveDialog',
            '$onInject',
            '$onImport',
            '$buttonReset'
        ],

        options: {
            fieldId: false
        },

        initialize: function (options) {
            this.parent(options);

            this.$Input    = null;
            this.$Data     = {};
            this.$Criteria = {};

            this.$Grid = null;

            this.addEvents({
                onInject: this.$onInject,
                onImport: this.$onImport
            });
        },

        /**
         * Create the DOMNode Element
         *
         * @return {HTMLDivElement}
         */
        create: function () {
            this.$Elm = new Element('div', {
                styles: {
                    'float': 'left',
                    width  : '100%'
                }
            });

            return this.$Elm;
        },

        /**
         * event : on inject
         */
        $onInject: function () {
            var Parent = this.$Elm.getParent('.field-options');

            if (Parent) {
                Parent.setStyle('padding', 0);
            }

            // Content
            const Container = new Element('div', {
                'class': 'quiqqer-affiliate-products-ratingtype-settings',
                html   : Mustache.render(template, {
                    titleBasicSettings        : QUILocale.get(lg, 'controls.AffiliateRatingTypeSettings.tpl.titleBasicSettings'),
                    labelTotalRatingType      : QUILocale.get(lg, 'controls.AffiliateRatingTypeSettings.tpl.labelTotalRatingType'),
                    optionRatingTypePercentage: QUILocale.get(lg, 'controls.AffiliateRatingTypeSettings.tpl.optionRatingTypePercentage'),
                    optionRatingTypePoints    : QUILocale.get(lg, 'controls.AffiliateRatingTypeSettings.tpl.optionRatingTypePoints'),
                    optionRatingTypeBoolean   : QUILocale.get(lg, 'controls.AffiliateRatingTypeSettings.tpl.optionRatingTypeBoolean'),
                    optionRatingTypeYesNo     : QUILocale.get(lg, 'controls.AffiliateRatingTypeSettings.tpl.optionRatingTypeYesNo'),
                    optionRatingTypeText      : QUILocale.get(lg, 'controls.AffiliateRatingTypeSettings.tpl.optionRatingTypeText'),
                    titleCriteria             : QUILocale.get(lg, 'controls.AffiliateRatingTypeSettings.tpl.titleCriteria'),
                    labelTotalRatingMaxPoints : QUILocale.get(lg, 'controls.AffiliateRatingTypeSettings.tpl.labelTotalRatingMaxPoints')
                })
            }).inject(this.$Elm);

            // Show / hide total rating max points
            const TotalRatingTypeSelect     = Container.getElement('[name="totalRatingType"]');
            const TotalRatingMaxPointsInput = Container.getElement('[name="totalRatingMaxPoints"]');

            if ('totalRatingType' in this.$Data) {
                TotalRatingTypeSelect.value = this.$Data.totalRatingType;
            }

            if ('totalRatingMaxPoints' in this.$Data) {
                TotalRatingMaxPointsInput.value = this.$Data.totalRatingMaxPoints;
            }

            TotalRatingTypeSelect.addEvent('change', () => {
                if (TotalRatingTypeSelect.value === '2') {
                    TotalRatingMaxPointsInput.disabled = false;
                    TotalRatingMaxPointsInput.focus();
                } else {
                    TotalRatingMaxPointsInput.disabled = true;
                }

                this.update();
            });

            TotalRatingMaxPointsInput.addEvent('change', this.update);

            if (TotalRatingTypeSelect.value === '2') {
                TotalRatingMaxPointsInput.disabled = false;
            }

            const self          = this;
            const GridContainer = this.$Elm.getElement('.quiqqer-affiliate-products-ratingtype-criteria');

            this.$Grid = new Grid(GridContainer, {
                perPage    : 150,
                buttons    : [{
                    name     : 'up',
                    textimage: 'fa fa-angle-up',
                    disabled : true,
                    events   : {
                        onClick: function () {
                            this.$moveup();
                            // this.$refreshSorting();
                        }.bind(this)
                    }
                }, {
                    name     : 'down',
                    textimage: 'fa fa-angle-down',
                    disabled : true,
                    events   : {
                        onClick: function () {
                            this.$movedown();
                            // this.$Grid.movedown();
                            // this.$refreshSorting();
                        }.bind(this)
                    }
                }, {
                    type: 'separator'
                }, {
                    name     : 'add',
                    textimage: 'fa fa-plus',
                    text     : QUILocale.get('quiqqer/system', 'add'),
                    events   : {
                        onClick: this.openAddDialog
                    }
                }, {
                    name     : 'edit',
                    textimage: 'fa fa-edit',
                    text     : QUILocale.get('quiqqer/system', 'edit'),
                    disabled : true,
                    events   : {
                        onClick: function () {
                            var selected = self.$Grid.getSelectedData();

                            if (selected.length) {
                                self.openEditDialog(selected[0].id);
                            }
                        }
                    }
                }, {
                    type: 'separator'
                }, {
                    name     : 'delete',
                    textimage: 'fa fa-trash',
                    text     : QUILocale.get('quiqqer/system', 'delete'),
                    disabled : true,
                    events   : {
                        onClick: this.openRemoveDialog
                    }
                }],
                columnModel: [{
                    header   : QUILocale.get('quiqqer/quiqqer', 'id'),
                    dataIndex: 'id',
                    dataType : 'string',
                    width    : 50
                }, {
                    header   : QUILocale.get('quiqqer/system', 'title'),
                    dataIndex: 'title',
                    dataType : 'string',
                    width    : 120
                }, {
                    header   : QUILocale.get(lg, 'controls.AffiliateRatingTypeSettings.tbl.ratingType'),
                    dataIndex: 'ratingType',
                    dataType : 'string',
                    width    : 150
                }, {
                    dataIndex: 'pos',
                    dataType : 'number',
                    hidden   : true
                }]
            });

            const ContainerSize = GridContainer.getSize();

            this.$Grid.setHeight(ContainerSize.y);
            this.$Grid.setWidth(ContainerSize.x);

            this.$Grid.addEvents({
                onClick   : this.$buttonReset,
                onDblClick: function () {
                    self.$buttonReset();

                    self.openEditDialog(
                        self.$Grid.getSelectedData()[0].id
                    );
                }
            });

            this.refresh();
        },

        /**
         * event : on import
         *
         * @param self
         * @param {HTMLInputElement} Node
         */
        $onImport: function (self, Node) {
            this.$Input = Node;
            this.$Elm   = this.create();

            try {
                this.$Data = JSON.decode(this.$Input.value);

                if (typeOf(this.$Data.criteria) !== 'object' && !this.$Data.criteria.length) {
                    this.$Criteria = {};
                } else {
                    this.$Criteria = this.$Data.criteria;
                }
            } catch (e) {
                console.error(this.$Input.value);
                console.error(e);
            }

            this.$Elm.wraps(this.$Input);
            this.$onInject();
        },

        /**
         * reset the buttons from the grid
         * disable or enable the buttons dependent on selected indices
         */
        $buttonReset: function () {
            var selected = this.$Grid.getSelectedIndices(),
                buttons  = this.$Grid.getButtons();

            var Up = buttons.filter(function (Button) {
                return Button.getAttribute('name') === 'up';
            })[0];

            var Down = buttons.filter(function (Button) {
                return Button.getAttribute('name') === 'down';
            })[0];

            var Edit = buttons.filter(function (Button) {
                return Button.getAttribute('name') === 'edit';
            })[0];

            var Delete = buttons.filter(function (Button) {
                return Button.getAttribute('name') === 'delete';
            })[0];

            if (selected.length === 1) {
                Edit.enable();
                Delete.enable();
                Up.enable();
                Down.enable();
                return;
            }

            Edit.disable();
            Up.disable();
            Down.disable();

            if (selected.length > 1) {
                Delete.enable();
                return;
            }

            Delete.disable();
        },

        /**
         * refresh the grid data dispaly
         */
        refresh: function () {
            const data        = [];
            const currentLang = QUILocale.getCurrent();

            let i = 0;

            for (let [id, Criterion] of Object.entries(this.$Criteria)) {
                let langTitle         = '---';
                const CriterionTitles = JSON.decode(Criterion.title);

                if (typeof CriterionTitles[currentLang] !== 'undefined') {
                    langTitle = CriterionTitles[currentLang];
                }

                let ratingTypeLabel = '---';

                switch (parseInt(Criterion.ratingType)) {
                    case 1:
                        ratingTypeLabel = QUILocale.get(lg, 'controls.AffiliateRatingTypeSettings.tpl.optionRatingTypePercentage');
                        break;

                    case 2:
                        ratingTypeLabel = QUILocale.get(lg, 'controls.AffiliateRatingTypeSettings.tpl.optionRatingTypePoints');
                        break;

                    case 3:
                        ratingTypeLabel = QUILocale.get(lg, 'controls.AffiliateRatingTypeSettings.tpl.optionRatingTypeBoolean');
                        break;

                    case 4:
                        ratingTypeLabel = QUILocale.get(lg, 'controls.AffiliateRatingTypeSettings.tpl.optionRatingTypeYesNo');
                        break;

                    case 5:
                        ratingTypeLabel = QUILocale.get(lg, 'controls.AffiliateRatingTypeSettings.tpl.optionRatingTypeText');
                        break;
                }

                if (typeof Criterion.pos === 'undefined') {
                    Criterion.pos = i;
                }

                data.push({
                    id        : id,
                    title     : langTitle,
                    ratingType: ratingTypeLabel,
                    pos       : Criterion.pos
                });

                i++;
            }

            // sort by pos
            data.sort(function (a, b) {
                return a.pos - b.pos;
            });

            this.$Grid.setData({
                data: data
            });
        },

        /**
         * dialogs
         */

        /**
         * Opens the add dialog
         */
        openAddDialog: function () {
            new QUIConfirm({
                title    : QUILocale.get(lg, 'controls.AffiliateRatingTypeSettings.tpl.add.title'),
                icon     : 'fa fa-plus',
                texticon : false,
                maxHeight: 400,
                maxWidth : 600,
                autoclose: false,
                events   : {
                    onOpen  : (Win) => {
                        Win.Loader.show();

                        const Content = Win.getContent();

                        Content.set('html', Mustache.render(templateCriterion, {
                            labelId             : QUILocale.get('quiqqer/quiqqer', 'id'),
                            labelTitle          : QUILocale.get('quiqqer/quiqqer', 'title'),
                            labelRatingType     : QUILocale.get(lg, 'controls.AffiliateRatingTypeSettings.tpl.labelRatingType'),
                            labelRatingMaxPoints: QUILocale.get(lg, 'controls.AffiliateRatingTypeSettings.tpl.labelRatingMaxPoints'),

                            optionRatingTypePercentage: QUILocale.get(lg, 'controls.AffiliateRatingTypeSettings.tpl.optionRatingTypePercentage'),
                            optionRatingTypePoints    : QUILocale.get(lg, 'controls.AffiliateRatingTypeSettings.tpl.optionRatingTypePoints'),
                            optionRatingTypeBoolean   : QUILocale.get(lg, 'controls.AffiliateRatingTypeSettings.tpl.optionRatingTypeBoolean'),
                            optionRatingTypeYesNo     : QUILocale.get(lg, 'controls.AffiliateRatingTypeSettings.tpl.optionRatingTypeYesNo'),
                            optionRatingTypeText      : QUILocale.get(lg, 'controls.AffiliateRatingTypeSettings.tpl.optionRatingTypeText')
                        }));

                        QUI.parse(Content).then(() => {
                            // Show / hide total rating max points
                            const RatingTypeSelect     = Content.getElement('[name="ratingType"]');
                            const RatingMaxPointsInput = Content.getElement('[name="ratingMaxPoints"]');

                            RatingTypeSelect.addEvent('change', () => {
                                if (RatingTypeSelect.value === '2') {
                                    RatingMaxPointsInput.disabled = false;
                                    RatingMaxPointsInput.focus();
                                } else {
                                    RatingMaxPointsInput.disabled = true;
                                }
                            });

                            Win.Loader.hide();
                        });
                    },
                    onSubmit: (Win) => {
                        var Form = Win.getContent().getElement('form');

                        this.add(QUIFormUtils.getFormData(Form)).then(function () {
                            Win.close();
                        }, function (errorMsg) {
                            QUI.getMessageHandler().then(function (MH) {
                                MH.addError(errorMsg);
                            });
                        });
                    }
                }
            }).open();
        },

        /**
         * opens the edit dialog
         *
         * @param {String} id - entry identifier
         */
        openEditDialog: function (id) {
            if (!(id in this.$Criteria)) {
                return;
            }

            var self = this,
                Data = this.$Criteria[id];

            var lgPrefix = 'fields.control.AffiliateRatingTypeSettings.';

            new QUIConfirm({
                title    : QUILocale.get(lg, lgPrefix + 'edit.title'),
                icon     : 'fa fa-edit',
                texticon : false,
                maxHeight: 400,
                maxWidth : 600,
                events   : {
                    onOpen  : function (Win) {
                        const Content = Win.getContent();

                        Content.set('html', Mustache.render(templateCriterion, {
                            labelId             : QUILocale.get('quiqqer/quiqqer', 'id'),
                            labelTitle          : QUILocale.get('quiqqer/quiqqer', 'title'),
                            labelRatingType     : QUILocale.get(lg, 'controls.AffiliateRatingTypeSettings.tpl.labelRatingType'),
                            labelRatingMaxPoints: QUILocale.get(lg, 'controls.AffiliateRatingTypeSettings.tpl.labelRatingMaxPoints'),

                            optionRatingTypePercentage: QUILocale.get(lg, 'controls.AffiliateRatingTypeSettings.tpl.optionRatingTypePercentage'),
                            optionRatingTypePoints    : QUILocale.get(lg, 'controls.AffiliateRatingTypeSettings.tpl.optionRatingTypePoints'),
                            optionRatingTypeBoolean   : QUILocale.get(lg, 'controls.AffiliateRatingTypeSettings.tpl.optionRatingTypeBoolean'),
                            optionRatingTypeYesNo     : QUILocale.get(lg, 'controls.AffiliateRatingTypeSettings.tpl.optionRatingTypeYesNo'),
                            optionRatingTypeText      : QUILocale.get(lg, 'controls.AffiliateRatingTypeSettings.tpl.optionRatingTypeText')
                        }));

                        const Form = Content.getElement('form');

                        QUIFormUtils.setDataToForm(Data, Form);

                        QUI.parse(Content).then(() => {
                            // Show / hide total rating max points
                            const RatingTypeSelect     = Content.getElement('[name="ratingType"]');
                            const RatingMaxPointsInput = Content.getElement('[name="ratingMaxPoints"]');

                            RatingTypeSelect.addEvent('change', () => {
                                if (RatingTypeSelect.value === '2') {
                                    RatingMaxPointsInput.disabled = false;
                                    RatingMaxPointsInput.focus();
                                } else {
                                    RatingMaxPointsInput.disabled = true;
                                }
                            });

                            if (RatingTypeSelect.value === '2') {
                                RatingMaxPointsInput.disabled = false;
                            }

                            Win.Loader.hide();
                        });
                    },
                    onSubmit: function (Win) {
                        Win.Loader.show();

                        const Form = Win.getContent().getElement('form');

                        self.edit(QUIFormUtils.getFormData(Form));
                        Win.close();
                    }
                }
            }).open();
        },

        /**
         * open remove dialog
         */
        openRemoveDialog: function () {
            var self = this,
                data = this.$Grid.getSelectedData();

            var indices = data.map(function (Criterion) {
                return Criterion.id;
            });

            if (!indices.length) {
                return;
            }

            new QUIConfirm({
                title      : QUILocale.get(lg, 'control.AffiliateRatingTypeSettings.remove.title'),
                icon       : 'fa fa-trash',
                texticon   : 'fa fa-trash',
                text       : QUILocale.get(lg, 'control.AffiliateRatingTypeSettings.remove.text'),
                information: indices.join(', '),
                maxHeight  : 300,
                maxWidth   : 450,
                events     : {
                    onSubmit: function () {
                        self.remove(indices);
                    }
                }
            }).open();
        },

        /**
         * entry move up
         */
        $moveup: function () {
            const Criterion = this.$Grid.getSelectedData()[0];

            if (Criterion.pos === 0) {
                return;
            }

            Criterion.pos -= 1;
            this.$Criteria[Criterion.id].pos = Criterion.pos;

            for (let id in this.$Criteria) {
                if (!this.$Criteria.hasOwnProperty(id)) {
                    continue;
                }

                if (id === Criterion.id || this.$Criteria[id].pos < Criterion.pos) {
                    continue;
                }

                this.$Criteria[id].pos += 1;
            }

            this.refresh();
            this.update();
        },

        /**
         * entry move down
         */
        $movedown: function () {
            const Criterion = this.$Grid.getSelectedData()[0];

            if (Criterion.pos === Object.getLength(this.$Criteria)) {
                return;
            }

            Criterion.pos += 1;
            this.$Criteria[Criterion.id].pos = Criterion.pos;

            for (let id in this.$Criteria) {
                if (!this.$Criteria.hasOwnProperty(id)) {
                    continue;
                }

                if (id === Criterion.id || this.$Criteria[id].pos > Criterion.pos) {
                    continue;
                }

                this.$Criteria[id].pos -= 1;
            }

            this.refresh();
            this.update();
        },

        /**
         * Set the data to the input
         */
        update: function () {
            this.$Input.value = JSON.encode({
                criteria            : this.$Criteria,
                totalRatingType     : this.$Elm.getElement('[name="totalRatingType"]').value,
                totalRatingMaxPoints: this.$Elm.getElement('[name="totalRatingMaxPoints"]').value,
            });
        },

        /**
         * Add a criterion
         *
         * @param {Object} Criterion - criterion data
         * @return {Promise}
         */
        add: function (Criterion) {
            return new Promise((resolve, reject) => {
                if (!Criterion.id) {
                    reject(
                        QUILocale.get(lg, 'fields.control.AffiliateRatingTypeSettings.error.empty_id')
                    );

                    return;
                }

                if (Criterion.id in this.$Criteria) {
                    reject(
                        QUILocale.get(lg, 'fields.control.AffiliateRatingTypeSettings.error.duplicate_id')
                    );

                    return;
                }

                this.$Criteria[Criterion.id] = Criterion;

                this.refresh();
                this.update();

                resolve();
            });
        },

        /**
         * Remove criteria
         *
         * @param {Array} ids - entry identifiers
         */
        remove: function (ids) {
            if (!this.$Grid) {
                return;
            }

            for (var i = 0, len = ids.length; i < len; i++) {
                delete this.$Criteria[ids[i]];
            }

            var pos = 0;

            for (var id in this.$Criteria) {
                if (!this.$Criteria.hasOwnProperty(id)) {
                    continue;
                }

                this.$Criteria[id].pos = pos++;
            }

            this.refresh();
            this.update();
        },

        /**
         * Edit field entry
         *
         * @param {Object} Criterion
         */
        edit: function (Criterion) {
            this.$Criteria[Criterion.id] = Criterion;

            this.refresh();
            this.update();
        }
    });
});
