<?php

namespace QUI\ERP\Products\Affiliate;

use QUI;
use Quiqqer\Engine\Collector;
use QUI\ERP\Products\Affiliate\Controls\AffiliateFields;
use DusanKasan\Knapsack\Collection;

/**
 * Class Events
 *
 * Main event handlers for quiqqer/affiliate-products.
 */
class Events
{
    /**
     * Runs the setup for affiliate products
     *
     * - import the default fields
     *
     * @param QUI\Package\Package $Package
     */
    public static function onPackageSetup(QUI\Package\Package $Package)
    {
        if ($Package->getName() === 'quiqqer/affiliate-products') {
            Setup::execute();
        }
    }

    /**
     * @param Collector $Collection
     * @param $Product
     */
    public static function onTemplateProductButtonsAfter(Collector $Collection, $Product)
    {
        $AffiliateFields = new AffiliateFields([
            'Product' => $Product
        ]);

//        $Collection->append(QUI\Control\Manager::getCSS());
        $Collection->append($AffiliateFields->create());
    }

    /**
     * @param Collector $Collection
     * @param $Product
     */
    public static function onTemplateProductButtonsEnd(Collector $Collection, $Product)
    {
        try {
            /* @var $Product QUI\ERP\Products\Product\Product */
            $AffiliateLink     = $Product->getField(AffiliateProducts::FIELD_AFFILIATE_LINK);
            $AffiliateLinkText = $Product->getField(AffiliateProducts::FIELD_AFFILIATE_LINK_TEXT);

            $affiliateLink     = $AffiliateLink->getValue();
            $affiliateLinkText = $AffiliateLinkText->getValue();
        } catch (\Exception $Exception) {
            return;
        }

        if (empty($affiliateLink) || empty($affiliateLinkText)) {
            return;
        }

        $Collection->append(
            '<a href="'.$affiliateLink.'" target="_blank" class="btn btn-primary">'.
            $affiliateLinkText.
            '</a>'
        );
    }

    /**
     * @param Collector $Collection
     * @param $Product
     */
    public static function onTemplateProductBegin(Collector $Collection, $Product)
    {
        $Collection->append('<style>.product-data-actionButtons-buyNow { display: none; }</style>');
    }

    /**
     * @param $View
     * @param Collection $Buttons
     * @param $ProductControl
     */
    public static function onQuiqqerProductsProductViewButtonsAfter(
        $View,
        Collection &$Buttons,
        $ProductControl
    ) {
        return;
        $Buttons = new Collection([]);
    }

}
